<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;
use SM\Util\Procedure\FctRefMvCcustoContrato;

class FindFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function findContrato($filtro)
    {

        $sql = " SELECT NRSEQCONTRATO,
                        NRCONTRATOFCT
                FROM (
                SELECT CONT.NRSEQCONTRATO,
                        CONT.IDATIVO,
                        CONT.NRCONTRATOFCT ||' - '|| PARC.NMSECUNDARIPARC AS NRCONTRATOFCT,
                        (SELECT COUNT(*)
                            FROM FCT_TRABSERVCONTRATO
                            WHERE NRSEQCONTRATO = CONT.NRSEQCONTRATO
                            AND NRORG         = CONT.NRORG) AS QTITENSTRABSERV
                    FROM FCT_CONTRATO CONT
                    INNER JOIN PARCNEGOCIO PARC
                    ON CONT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findHabilidades($filtro)
    {

        $sql = "SELECT CDHABILITSERV,
                        NMHABILITSERV
                FROM    FCT_HABILITSERV
                WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findEscopoFaturamento($filtro)
    {

        $sql = " SELECT * FROM (
                    SELECT TIPO.NRTIPOORCAMENTO,
                            TIPO.NMTIPOORCAMENTO,
                            TIPO.NRORG,
                            TIPO.IDATIVO,
                            TIPO.IDFATURAORCA,
                            TIPO.IDAPURAFATUR,
                            TIPO.NRGRUPFATADIC,
                            TIPO.NRGRUPFATSERV,
                            TIPO.CDPRECBASEVEND,
                            TIPO.CDMINEMORCA,
                            TIPO.CDPRECBASESERV,
                            TIPO.DTINCLUSAO,
                            TIPO.DTULTATU,
                            TIPO.NRORGINCLUSAO,
                            TIPO.CDOPERINCLUSAO,
                            TIPO.NRORGULTATU,
                            TIPO.CDOPERULTATU,
                            MODC.NMMODCONTRATO,
                            GRUF.NMGRUPOFATURA,
                            PRECOBASE.DSPRECOBASE,
                            PBS.DSPRECBASESERV,
                            INC.NMOPERADOR AS NMOPERINCLUSAO,
                            ALT.NMOPERADOR AS NMOPERULTATU,
                            TIPO.IDUTILIZAVARPRECO,
                            TIPO.IDALTQTDMEDIDAPROP,
                            TIPO.IDALTVRTABRPECPROP,
                            TIPO.IDTIPOORCA,
                            TIPO.IDFILTRASERVICO,
                            NVL(
                                (SELECT DISTINCT 'S'
                                FROM FCT_RELACESCOPOAGRUPAMENTO
                                WHERE NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                                AND NRORG = TIPO.NRORG), 'N') AS ESCOPOAGRUPADO
                    FROM    FCT_TIPOORCAMENTO TIPO
                            LEFT JOIN FCT_MODELOCONTRATO MODC ON TIPO.NRSEQMODCONTRATO = MODC.NRSEQMODCONTRA
                            AND TIPO.NRORG = MODC.NRORG
                            LEFT JOIN FCT_GRUPOFATURA GRUF ON TIPO.NRGRUPFATSERV = GRUF.NRGRUPOFATURA
                            AND TIPO.NRORG = GRUF.NRORG
                            LEFT JOIN PRECOBASE ON TIPO.CDPRECBASEVEND = PRECOBASE.CDPRECOBASE
                            AND TIPO.NRORG = PRECOBASE.NRORG
                            LEFT JOIN FCT_PRECBASESERV PBS ON TIPO.CDPRECBASESERV = PBS.CDPRECBASESERV
                            AND TIPO.NRORG = PBS.NRORG
                            LEFT JOIN OPERADOR INC ON TIPO.CDOPERINCLUSAO = INC.CDOPERADOR
                            AND TIPO.NRORG = INC.NRORG
                            LEFT JOIN OPERADOR ALT ON TIPO.CDOPERULTATU = ALT.CDOPERADOR
                            AND TIPO.NRORG = ALT.NRORG 
                )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findModeloContrato($filtro)
    {

        $sql = "SELECT MODELO.NRSEQMODCONTRA,
                        MODELO.CDMODCONTRATO,
                        MODELO.NMMODCONTRATO,
                        MODELO.CDINDFINCONT,
                        MODELO.CDTPINDFINCON,
                        MODELO.IDPERVALFINCON,
                        MODELO.NRDIAVENPARCFAT,
                        MODELO.NRDIASVIGCONT,
                        MODELO.DTPADREAJCONT,
                        MODELO.NRDIASPRORGCONT,
                        MODELO.QTPOSTOTRABCONT,
                        MODELO.VRPERCMULTQBCONT,
                        MODELO.NRORG,
                        MODELO.DTINCLUSAO,
                        MODELO.DTULTATU,
                        MODELO.NRORGINCLUSAO,
                        MODELO.CDOPERINCLUSAO,
                        MODELO.NRORGULTATU,
                        MODELO.CDOPERULTATU,
                        MODELO.IDATIVO,
                        MODELO.IDTPCOMPETCONT,
                        MODELO.NRDIACOMPETCONT,
                        MODELO.IDTPDTEMISSNF,
                        MODELO.NRDIADTEMISSNF,
                        MODELO.IDTPDTVENC,
                        MODELO.IDQTDIASCOMPCON,
                        MODELO.NRQTDIASCOMPCON,
                        MODELO.IDTPREAJUSTE,
                        MODELO.IDTPMESREAJUSTE,
                        MODELO.NRQTDREAJUSTE,
                        MODELO.IDTPFATUINCOMP,
                        MODELO.IDSOMAFATUCONT,
                        MODELO.IDBLOQSEMMEDIC,
                        MODELO.IDBLOQVRMEDIC,
                        MODELO.IDFATURACONTR,
                        MODELO.CDMODPGTO,
                        MODELO.DTINICFATCONTR,
                        MODELO.QTPARCELASFATU,
                        MODELO.NRDIAVENCPARCELASFATU,
                        MODELO.IDGERAPROPOSTA,
                        MODELO.IDRENOVACAOAUTOMATICA,
                        MODELO.IDREAJUSTEAUTOMATICO,
                        MODELO.IDASSELETRDIGITAL,
                        MODELO.IDBLOQALTDTINICONTRENEG,
                        MODELO.NRDIASREAJAUTOMATICO,
                        MODELO.IDENVIADOCOPERRESPCONCASS,
                        MODELO.CDCENTCUST,
                        CENTCUST.NMCENTCUST
                FROM FCT_MODELOCONTRATO MODELO
                LEFT JOIN CENTCUST ON MODELO.CDCENTCUST = CENTCUST.CDCENTCUST
                AND MODELO.NRORG = CENTCUST.NRORG
                WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findParceiroNegocio($filtro, $params)
    {

        $sql = "SELECT * FROM (
                    SELECT DISTINCT PARCNEGOCIO.NRPARCNEGOCIO,
                                    PARCNEGOCIO.NMPRINCIPALPARC,
                                    PARCNEGOCIO.NMSECUNDARIPARC,
                                    PARCNEGOCIO.NRINSCRICAOPARC,
                                    PARCNEGOCIO.CDTIPOPARCPRINCIPAL,
                                    PARCNEGOCIO.NRORG,
                                    PARCNEGOCIO.CDTIPOINSCRICAO,
                                    DECODE(PARCNEGOCIO.IDATIVO, 'S', 'Sim', 'Não') AS IDATIVO,
                                    OPERADOR.NMOPERADOR AS NMOPERRESPONSAVEL,
                                    PARCNEGOCIO.NMPRINCIPALPARC || ' | ' || PARCNEGOCIO.NRINSCRICAOPARC || ' | ' || PARCNEGOCIO.CDTIPOPARCPRINCIPAL AS GRIDPARCFILTER
                            FROM PARCNEGOCIO
                            LEFT JOIN PARCERIA ON PARCNEGOCIO.NRPARCNEGOCIO = PARCERIA.NRPARCNEGOCIO
                            AND PARCNEGOCIO.NRORG = PARCERIA.NRORG
                            LEFT JOIN OPERADOR ON PARCNEGOCIO.CDOPERRESPONSAVEL = OPERADOR.CDOPERADOR
                            AND PARCNEGOCIO.NRORG = OPERADOR.NRORG
                            LEFT JOIN
                            (SELECT *
                            FROM CRM_HIERARQUIAOPER
                            WHERE CDOPERADOR = :P_CDOPERADOR ) OPERHIERAR ON OPERHIERAR.NRORG = PARCNEGOCIO.NRORG
                            LEFT JOIN FCT_PARAMETROGERAL PARAM ON PARCNEGOCIO.NRORG = PARAM.NRORG
                            WHERE (PARCERIA.CDTIPOPARCEIRO IN('CLIENTE', 'PROSPECT'))
                            AND ((PARAM.IDUTILIZAHIEROPER = 'S'
                            AND (PARCNEGOCIO.CDOPERRESPONSAVEL IN
                                (SELECT CDOPERADOR
                                    FROM CRM_HIERARQUIAOPER
                                    WHERE (CRM_HIERARQUIAOPER.CDFILIAL = OPERHIERAR.CDFILIAL
                                        OR CRM_HIERARQUIAOPER.CDFILIAL IS NULL) CONNECT BY
                                    PRIOR CRM_HIERARQUIAOPER.NRHIERARQUIAOPER = CRM_HIERARQUIAOPER.NRHIERARQUIAPAI
                                    START WITH CRM_HIERARQUIAOPER.CDOPERADOR = OPERHIERAR.CDOPERADOR ))
                            OR PARCNEGOCIO.CDOPERRESPONSAVEL IS NULL)
                            OR (OPERHIERAR.IDOPERMASTER = 'S')
                            OR (PARAM.IDUTILIZAHIEROPER = 'N'))
                )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function findOperadorHierarquia($filtro, $params) {

        $sql = "SELECT * FROM (
            SELECT *
        FROM (
        SELECT OPER.CDOPERADOR,
               UPPER(OPER.NMOPERADOR) AS NMOPERADOR,
               OPER.NRPARCNEGOCIO,
               OPER.NRORG,
               DECODE(OPER.IDATIVO, 'S', 'Sim', 'Não') AS IDATIVO
        FROM OPERADOR OPER
        LEFT JOIN
          (SELECT *
           FROM CRM_HIERARQUIAOPER CRMH
           WHERE CDOPERADOR =:CDOPERADOR ) HIERARPAI ON OPER.NRORG = HIERARPAI.NRORG
        INNER JOIN FCT_PARAMETROGERAL PARAM ON OPER.NRORG = PARAM.NRORG
        WHERE (PARAM.IDUTILIZAHIEROPER = 'N')
          OR (PARAM.IDUTILIZAHIEROPER = 'S'
              AND (OPER.CDOPERADOR IN
                     (SELECT :CDOPERADOR
                      FROM DUAL
                      UNION ALL SELECT CDOPERADOR
                      FROM CRM_HIERARQUIAOPER
                      WHERE (CRM_HIERARQUIAOPER.CDFILIAL = HIERARPAI.CDFILIAL
                             OR CRM_HIERARQUIAOPER.CDFILIAL IS NULL) CONNECT BY
                        PRIOR CRM_HIERARQUIAOPER.NRHIERARQUIAOPER = CRM_HIERARQUIAOPER.NRHIERARQUIAPAI
                        START WITH CRM_HIERARQUIAOPER.CDOPERADOR = HIERARPAI.CDOPERADOR ))
              OR OPER.CDOPERADOR IS NULL
              OR OPER.CDOPERADOR =:CDOPERADOR)
          OR (HIERARPAI.IDOPERMASTER = 'S')) 
          )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function findIdioma($filtro)
    {
        $sql = "SELECT * FROM IDIOMA WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findMoeda($filtro)
    {
        $sql = "SELECT * FROM FCT_MOEDA WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findProposta($filtro, $params)
    {

        $sql = "SELECT * FROM (
             SELECT ORC.NRSEQORCAMENTO,
                    ORC.NRORCAMENTO || ' - ' || ORC.DSREFERENCIAORCA AS NMPROPOSTA,
                    ORC.NRORG,
                    ORC.NRORCAMENTO,
                    ORC.NRVERSAOORCAMENTO,
                    ORC.IDSITUACAOORCAMENTO,
                    ORC.DTREFORCAMENTO,
                    ORC.DTACEITEORCA,
                    ORC.NRORCAMENTOMODELO,
                    ORC.CDOPERRESPONSAVEL,
                    OPER.NMOPERADOR AS NMOPERRESPONSAVEL,
                    ORC.NRPARCNEGOCIO,
                    PARC.NMPRINCIPALPARC,
                    MODELO.IDUTILIZACOMP,
                    ORC.DSREFERENCIAORCA,
                    ORC.VRORCAMENTO,
                    ORC.IDATIVO,
                    ORC.DTREFORCAMENTO AS DTINIPROPOSTA,
                    ORC.DTVALIDADE AS DTFINPROPOSTA,
                    ORC.NRORCAMENTO || ' | ' || ORC.DSREFERENCIAORCA || ' | ' || ORC.DTREFORCAMENTO || ' | ' || PARC.NMPRINCIPALPARC AS PROPOSTAFILTER,
                (SELECT LISTAGG(ESCOPO.IDUTILIZAVARPRECO, ',') WITHIN GROUP(
                                                                            ORDER BY ESCOPO.NMTIPOORCAMENTO)
                FROM FCT_TIPOORCAMENTO ESCOPO
                INNER JOIN FCT_RELACESCOPOPROPOSTA REL ON ESCOPO.NRTIPOORCAMENTO = REL.NRTIPOORCAMENTO
                AND ESCOPO.NRORG = REL.NRORG
                WHERE REL.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND REL.NRORG = ORC.NRORG ) AS IDUTILIZAVARPRECO,

                (SELECT LISTAGG(ESCOPO.NRTIPOORCAMENTO, ',') WITHIN GROUP(
                                                                            ORDER BY ESCOPO.NMTIPOORCAMENTO)
                FROM FCT_TIPOORCAMENTO ESCOPO
                INNER JOIN FCT_RELACESCOPOPROPOSTA REL ON ESCOPO.NRTIPOORCAMENTO = REL.NRTIPOORCAMENTO
                AND ESCOPO.NRORG = REL.NRORG
                WHERE REL.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                    AND REL.NRORG = ORC.NRORG ) AS NRTIPOORCAMENTO
                FROM FCT_ORCAMENTO ORC
                LEFT JOIN PARCNEGOCIO PARC ON ORC.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                AND ORC.NRORG = PARC.NRORG
                LEFT JOIN FCT_ORCAMENTOMODELO MODELO ON ORC.NRORCAMENTOMODELO = MODELO.NRORCAMENTOMODELO
                AND ORC.NRORG = MODELO.NRORG
                LEFT JOIN OPERADOR OPER ON ORC.CDOPERRESPONSAVEL = OPER.CDOPERADOR
                AND ORC.NRORG = OPER.NRORG
                LEFT JOIN
                (SELECT NRHIERARQUIAOPER,
                        NRHIERARQUIAPAI,
                        NRORG,
                        CDOPERADOR,
                        IDOPERMASTER,
                        CDFILIAL
                FROM CRM_HIERARQUIAOPER
                WHERE CDOPERADOR =:CDOPERADOR ) HIERARPAI ON HIERARPAI.NRORG = ORC.NRORG
                LEFT JOIN CRM_HIERARQUIAOPER HIERAR ON ORC.CDOPERRESPONSAVEL = HIERAR.CDOPERADOR
                AND ORC.NRORG = HIERAR.NRORG
                LEFT JOIN FCT_PARAMETROGERAL PARAM ON PARAM.NRORG = ORC.NRORG
                WHERE (PARAM.IDUTILIZAHIEROPER = 'N')
                OR (PARAM.IDUTILIZAHIEROPER = 'S'
                    AND (ORC.CDOPERRESPONSAVEL IN
                            (SELECT CDOPERADOR
                            FROM CRM_HIERARQUIAOPER
                            WHERE (CRM_HIERARQUIAOPER.CDFILIAL = HIERARPAI.CDFILIAL
                                    OR CRM_HIERARQUIAOPER.CDFILIAL IS NULL) CONNECT BY
                                PRIOR CRM_HIERARQUIAOPER.NRHIERARQUIAOPER = CRM_HIERARQUIAOPER.NRHIERARQUIAPAI
                                START WITH CRM_HIERARQUIAOPER.CDOPERADOR = HIERARPAI.CDOPERADOR ))
                    OR ORC.CDOPERRESPONSAVEL IS NULL)
                OR (HIERARPAI.IDOPERMASTER = 'S')
                AND EXISTS
                    (SELECT 1
                    FROM FCT_REFEORCACONT
                    WHERE NRSEQORCAMENTO = ORC.NRSEQORCAMENTO)
        )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function findModeloProposta($filtro)
    {

        $sql = "SELECT * FROM (
                     SELECT ORC.NRORCAMENTOMODELO,
                            ORC.NRARVOREPAI,
                            ORC.NRTIPOORCAMENTO,
                            ORC.NRPARCTRANSPORTADORA,
                            ORC.CDREDUCONT,
                            ORC.NMORCAMENTOMODELO,
                            ORC.DSREFERENCIAORCA,
                            ORC.IDTRANSPORTADORA,
                            ORC.QTDIASVALIDADE,
                            ORC.QTDIASCONTRATO,
                            ORC.DSOBSERVACAOORCA,
                            ORC.CDMINEMORCA,
                            ORC.IDFATURACONTR,
                            ORC.CDMODPGTO,
                            ORC.NRGRUPOFATURA,
                            ORC.IDUTILIZAHIERSERV,
                            ORC.IDBLOQALTERADADOS,
                            ORC.IDOBRIGAOPORTUNIDADE,
                            ORC.IDUTILIZACOMP,
                            ORC.NRDIASCARENCIA,
                            TIPO.NMTIPOORCAMENTO,
                            GRUPO.NMGRUPOFATURA,
                            MODPGTO.DSMODPGTO,
                            ORC.NRORG,
                            ORC.IDATIVO,
                            ORC.DTINCLUSAO,
                            ORC.DTULTATU,
                            ORC.NRORGINCLUSAO,
                            ORC.CDOPERINCLUSAO,
                            ORC.NRORGULTATU,
                            ORC.CDOPERULTATU,
                            ORC.QTPARCELASFATU,
                            ORC.NRDIAVENCPARCELASFATU,
                            ORC.NRSEQMODCONTRA,
                            MODCONT.NMMODCONTRATO,
                            ORC.IDASSELETRDIGITAL,
                            ORC.IDENVIADOCCONCLUSAO,
                            ORC.IDENVIADOCOPERRESPCONCASS,
                            ORC.NRTIPODOCCERTSIGN,
                            TIPODOC.NMTIPODOCCERTSIGN
                        FROM FCT_ORCAMENTOMODELO ORC
                        LEFT JOIN FCT_TIPOORCAMENTO TIPO ON ORC.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                        AND ORC.NRORG = TIPO.NRORG
                        LEFT JOIN FCT_GRUPOFATURA GRUPO ON ORC.NRGRUPOFATURA = GRUPO.NRGRUPOFATURA
                        AND ORC.NRORG = GRUPO.NRORG
                        LEFT JOIN MODPGTO ON ORC.CDMODPGTO = MODPGTO.CDMODPGTO
                        AND ORC.NRORG = MODPGTO.NRORG
                        LEFT JOIN FCT_MODELOCONTRATO MODCONT ON ORC.NRSEQMODCONTRA = MODCONT.NRSEQMODCONTRA
                        AND ORC.NRORG = MODCONT.NRORG
                        LEFT JOIN FCT_TIPODOCCERTSIGN TIPODOC ON ORC.NRTIPODOCCERTSIGN = TIPODOC.NRTIPODOCCERTSIGN
                        AND ORC.NRORG = TIPODOC.NRORG
                )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findCentroCusto($filtro, $params)
    {

        $sql = "SELECT * FROM (
                    SELECT DISTINCT CENTCUST.CDCENTCUST,
                                    CENTCUST.NMCENTCUST,
                                    CENTCUST.NRORG
                    FROM CENTCUST
                    JOIN CENTCUSTFIL ON CENTCUSTFIL.CDCENTCUST = CENTCUST.CDCENTCUST
                    AND CENTCUSTFIL.NRORG = CENTCUST.NRORG
                    WHERE CDFILIAL = :CDFILIAL
                    OR :CDFILIAL IS NULL
                )WHERE " . $filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function findServico($filtro) {
        $sql = "SELECT * FROM FCT_SERVICOORCA WHERE ".$filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findTipoMedida($filtro) {
        $sql = "SELECT * FROM FCT_TIPOMEDIDASERV WHERE ".$filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findOperador($filtro) {
        $sql = "SELECT * FROM OPERADOR WHERE ".$filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function findPerfilAcesso($filtro) {
        $sql = "SELECT * FROM PERFILACESSO WHERE ".$filtro;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

}
